import re
from difflib import SequenceMatcher

tokenizer = re.compile(r'<[a-zA-Z][-_.:a-zA-Z0-9]*/?\s*(?:'
        r'\s*(?:[a-zA-Z_][-:.a-zA-Z_0-9]*)(?:\s*=\s*'
        r'(?:\'[^\']*\'|"[^"]*"|[-a-zA-Z0-9./,:;+*%?!&$\(\)_#=~@]'
        r'[][\-a-zA-Z0-9./,:;+*%?!&$\(\)_#=~\'"@]*(?=[\s>/<])))?'
    r')*\s*/?\s*>|.')

class Tag:
  def __init__(self,string):
    self.string = string
  def __len__(self):
    return 1
  def isspace(self):
    return False

def wrap(string):
  if len(string)>1: return Tag(string)
  return string

def join(list):
  result = ''
  for token in list:
    result += str(token)
  return result

def junk(token): return len(token)>1

class diff:
    def __init__(self,before, after):
        self.before = map(wrap,tokenizer.findall(before))
        self.after = map(wrap,tokenizer.findall(after))
        self.result = ''
        for op,s1,e1,s2,e2 in SequenceMatcher(junk,before,after).get_opcodes():
            getattr(self,op)(s1,e1,s2,e2)

    def equal(self, s1, e1, s2, e2):
        self.result += join(self.before[s1:e1])

    def insert(self, s1, e1, s2, e2):
        if self.after[s2].isspace():
            self.result+=self.after[s2]
            if e2 > s2+1:
                self.insert(s1, e1, s2+1, e2)
        elif self.after[e2-1].isspace():
            if e2-1 > s2:
                self.insert(s1, e1, s2, e2-1)
            self.result+=self.after[e2-1]
        else:
            self.result += '<ins>' + join(self.after[s2:e2]) + '</ins>'
  
    def delete(self, s1, e1, s2, e2):
        if self.before[s1].isspace():
            if e1 > s1+1:
                self.delete(s1+1, e1, s2, e2)
        elif self.before[e1-1].isspace():
            if e1-1 > s1:
                self.delete(s1, e1-1, s2, e2)
        else:
            self.result += '<del>' + join(self.before[s1:e1]) + '</del>'

    def replace(self, s1, e1, s2, e2):
        self.delete(s1, e1, s2, e2)
        self.insert(s1, e1, s2, e2)
  
if __name__ == '__main__':
    assert(diff('foo bar', 'foo bar').result == 'foo bar')
    assert(diff('ac', 'abc').result == 'a<ins>b</ins>c')
    assert(diff('a c', 'a b c').result == 'a <ins>b</ins> c')
    assert(diff('   ', '      ').result == '      ')
    assert(diff('     ', '    ').result == '    ')
    assert(diff(u'before <a href="foo">text</a> after',
                u'before <a href="bar">text</a> after').result,
                u'before <a href="bar">text</a> after')
    print diff('a b c', 'a <b>b</b> c').result
    assert(diff('a b c', 'a <b>b</b> c').result == 'a <b>b</b> c')
