from BaseHTTPServer import HTTPServer, BaseHTTPRequestHandler
import planet.server_debug
import base64, cgi, os, time, sys, socket
from mimetypes import guess_type

class PlanetHandler(BaseHTTPRequestHandler):
    def __init__(self, request, client_address, server):
        self.main = server.main
        self.template_file = self.main.template_files[0]

        self.encoding = self.main.my_planet.tmpl_config_get(
            self.template_file, "encoding", planet.ENCODING)

        self.output_dir = self.main.my_planet.tmpl_config_get(
            self.template_file, "output_dir", planet.OUTPUT_DIR)

        user = self.main.my_planet.tmpl_config_get("Planet", "server_user")
        paswd = self.main.my_planet.tmpl_config_get("Planet", "server_pswd")

        if user and pswd:
            self.auth = 'Basic ' + base64.encodestring(user+':'+pswd)[:-1]
        else:
            self.auth = None

        BaseHTTPRequestHandler.__init__(self, request, client_address, server)

    def do_GET(self):
        realpath = self.path[1:]
        if realpath.startswith('feed/'): realpath=realpath[5:]
        realpath=os.path.join(self.output_dir,realpath)

        if os.path.isfile(realpath):
            mimetype = guess_type(realpath) or 'application/octet-stream'
            modified = time.gmtime(os.stat(realpath).st_mtime)
            modified = time.strftime(planet.TIMEFMT_822, modified)
            if self.headers.get('If-Modified-Since','NONE') == modified:
                self.send_response(304)
            else:
                self.send_response(200)
                size = os.stat(realpath).st_size
                self.send_header('Content-Type', mimetype)
                self.send_header('Content-Length', os.stat(realpath).st_size)
                self.send_header('Last-Modified', modified)
                self.end_headers()
                self.wfile.write(open(realpath).read())
            return

        if self.path.startswith('/feed/'):
            self.feed()
        elif '/feed/'.startswith(self.path):
            self.send_response(302)
            self.send_header('Location', '/feed/')
            self.end_headers()
        elif self.main.verbose and self.path.startswith('/debug/'):
            reload(planet.server_debug).run(self)
        else:
            self.send_error(404)

    def do_POST(self):

        if self.auth:
            if self.headers.get('authorization') != self.auth:
                realm = self.main.planet_name
                self.send_response(401)
                self.send_header('WWW-Authenticate', 'Basic realm="%s"' % realm)
                self.end_headers()
                return
            
        length = int(self.headers.getheader('content-length'))
        query = dict(cgi.parse_qsl(self.rfile.read(length)))
        if 'url' in query:
            feed = query['url']
            self.path = planet.cache.filename('/feed/', feed)
            if not self.main.my_planet.config.has_section(feed):
                self.main.my_planet.config.add_section(feed)
                self.main.my_planet.config.set(feed, 'name', '*** preview ***')
                channel = planet.Channel(self.main.my_planet, feed)
                self.main.my_planet.subscribe(channel)
                channel.update()
                self.do_GET()
                self.main.my_planet.unsubscribe(channel)
        else:
            self.do_GET()

    # produce results for a single feed
    def feed(self):
        # resynch with cache
        self.main.my_planet.freshen()

        # find feed
        cache_filename = self.path.split('/')[-1]
        channels = self.main.my_planet.gather_channel_info()[0]
        channel = self.main.my_planet.find_by_basename(cache_filename)
        if channel:
            items = self.main.my_planet.gather_items_info(channels,
                channel_list=[channel])
            admin = open('planet/server/feed.partial').read()
        else:
            items = self.main.my_planet.gather_items_info(channels,
                channel_list=channels.keys())
            admin = open('planet/server/admin.partial').read()

        # construct template
        date_format = self.main.my_planet.tmpl_config_get("Planet",
            "date_format",planet.DATE_FORMAT)

        manager = planet.htmltmpl.TemplateManager()
        template = manager.prepare(self.template_file)
        tp = planet.htmltmpl.TemplateProcessor(html_escape=0)
        tp.set('name', self.main.planet_name)
        tp.set('generator', planet.VERSION)
        tp.set('date', time.strftime(date_format, time.gmtime()))

        cache_directory = self.main.my_planet.cache_directory
        channel_info = self.main.my_planet.gather_channel_info()[1]
        for i in range(0,len(channel_info)):
            channel_info[i]['link'] = \
                planet.cache.filename('', channel_info[i]['url'])
        tp.set('Channels', channel_info)
        tp.set('Items', items)
        tp.set('admin', admin)

        # produce response
        self.send_response(200)
        self.send_header('Content-type', 'text/html; charset='+self.encoding)
        self.end_headers()

        self.wfile.write(tp.process(template))


# based on http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/442473
class PlanetHTTPServer(HTTPServer):
    def __init__(self, vars):
        my_planet = vars['my_planet']

        cert = my_planet.tmpl_config_get("Planet", "server_cert")
        port = my_planet.tmpl_config_get("Planet", "server_port",
            (cert and 443 or 80))

        HTTPServer.__init__(self, ('',int(port)), PlanetHandler)
        if cert: self.setcert(cert)

        class main: pass
        main.__dict__.update(vars)
        self.main = main

        user = self.main.my_planet.tmpl_config_get("Planet", "server_user")
        pswd = self.main.my_planet.tmpl_config_get("Planet", "server_pswd")
        if user and pswd:
            self.auth = 'Basic ' + \
                base64.encodestring('%s:%s' % (user, pswd))[:-1]
        else:
            self.auth = None

        server, port = self.socket.getsockname()
        if cert:
            main.log.info('Started HTTPS server on %s port %d', server, port)
        else:
            main.log.info('Started HTTP server on %s port %d', server, port)

    def setcert(self, fpem):
        from OpenSSL import SSL

        # OpenSSH's Connection does not implement a 'makefile' method, nor does
        # that class allow subclassing, so we add one via delegation.
        class SSLConnectionProxy:
            def __init__(self, context, socket):
                if context:
                    self.realConnection = SSL.Connection(context, socket)
                else:
                    self.realConnection = socket
            def __getattr__(self, name):
                if hasattr(self.realConnection, name):
                    return getattr(self.realConnection, name)
                else:
                    raise AttributeError, name
            def accept(self):
                connection, addr = self.realConnection.accept()
                return SSLConnectionProxy(None, connection), addr
            def makefile(self, mode, bufsize):
               return socket._fileobject(self.realConnection, mode, bufsize)

        ctx = SSL.Context(SSL.SSLv23_METHOD)
        ctx.use_privatekey_file(fpem)
        ctx.use_certificate_file(fpem)
        self.socket = SSLConnectionProxy(ctx,
            socket.socket(self.address_family, self.socket_type))
        self.server_bind()
        self.server_activate()
