#!/usr/bin/env python

import unittest
import planet.truncate

class TruncateTest(unittest.TestCase):
  def assertT(self, source, n, target):
    t = planet.truncate._TruncateHTMLProcessor(n)
    t.feed(source)
    t.close()
    self.assertEqual(t.output(), target)
    
  def test_null(self):
    self.assertT("foo", 50, "foo")

  def test_foos(self):
    self.assertT("foo " * 80, 18, 'foo foo foo foo&hellip;')
    self.assertT("foo " * 80, 19, 'foo foo foo foo foo&hellip;')
    self.assertT("foo " * 80, 20, 'foo foo foo foo foo&hellip;')
    self.assertT("foo " * 80, 21, 'foo foo foo foo foo&hellip;')
    self.assertT("foo " * 80, 22, 'foo foo foo foo foo&hellip;')
    self.assertT("foo " * 80, 23, 'foo foo foo foo foo foo&hellip;')

  def test_spaces(self):
    self.assertT("foo     foo", 40, 'foo     foo')

  def test_nesting(self):
    self.assertT("<b>"*10+'x'+"</b>"*10, 10,
                 "<b>"*10+'x'+"</b>"*10)
    self.assertT("<b>"*10+'foo foo'+"</b>"*10, 5,
                 "<b>"*10+'foo&hellip;'+"</b>"*10)
    self.assertT("<b>"*10+'foo foo'+"<b>"*10+'bar'+"</b>"*20, 5,
                 "<b>"*10+'foo&hellip;'+"</b>"*10)

  def test_link(self):
    self.assertT('before <a href="http://example.com">link</a> after', 17,
                 'before <a href="http://example.com">link</a> after')
    self.assertT('before <a href="http://example.com">link</a> after', 16,
                 'before <a href="http://example.com">link</a>&hellip;')
    self.assertT('before <a href="http://example.com">link</a> after', 11,
                 'before <a href="http://example.com">link</a>&hellip;')
    self.assertT('before <a href="http://example.com">link</a> after', 10,
                 'before <a href="http://example.com">&hellip;</a>')

  def test_image(self):
    self.assertT('pic: <img src="cat.jpg">.', 20,
                 'pic: <a href="cat.jpg">&lt;img&gt;</a>.')
    self.assertT('pic: <img src="http://example.com/cat.jpg" alt="">.', 20,
                 'pic: .')
    self.assertT('pic: <img src="cat.jpg" alt="cat">.', 20,
                 'pic: <a href="cat.jpg">cat</a>.')
    self.assertT('pic: <img src="cat.jpg" alt="cat">.', 5,
                 'pic: <a href="cat.jpg">&hellip;</a>')
    self.assertT('pic: <img src="cat.jpg?a=1&amp;b=2" alt="cat">.', 5,
                 'pic: <a href="cat.jpg?a=1&amp;b=2">&hellip;</a>')
    self.assertT('pic: <a href="bigcat.jpg"><img src="cat.jpg" alt="cat"></a>.',
                 10, 'pic: <a href="bigcat.jpg">cat</a>.')

  def test_refs(self):
    self.assertT('can&rsquo;t '*30, 10, 'can&rsquo;t can&rsquo;&hellip;')
    self.assertT('can&rsquo;t '*30, 9, 'can&rsquo;t can&rsquo;&hellip;')
    self.assertT('can&rsquo;t '*30, 8, 'can&rsquo;t&hellip;')

    self.assertT('can&#x2019;t '*30, 10, 'can&#x2019;t can&#x2019;&hellip;')
    self.assertT('can&#x2019;t '*30, 9, 'can&#x2019;t can&#x2019;&hellip;')
    self.assertT('can&#x2019;t '*30, 8, 'can&#x2019;t&hellip;')

if __name__ == '__main__':
    unittest.main()
